#!/bin/sh
#
# Cross-correlation of noise data.
# Script finds all possible combinations.
# See 1_README_example_pcc5h for further informations
#
# 19/11/2016, schimmel@ictja.csic.es
#--------------------------------------

# Executables:
#-------------
pcc="../../pcc5iMP"
norm="../../norm_tavg_white"

# Frequency band pass:
#---------------------
bp1=1.0
bp2=2.0

# Working dir:
#-------------
dir0=`pwd `

# Loop over all data:
#--------------------
for i in 2009.???.??
do
dir=$dir0"/"$i

### Alternative strategy to define loop:
###for i in `seq -w 1 366`
###do
###dir=$dir0"/2009."$i".00"

# if data dir exists then goto data dir: 
#---------------------------------------
if test -d $dir
then
cd $dir
echo $dir
nl=` ls -1 *SAC | wc -l | tail -1 | awk '{ if ($1 > 1) print "2" }' `
else
nl=1
fi

#process if nl=2:
#----------------
## alternative in case problems with if:
##  if [ "$nl" -eq "2" ]
if [ "$nl" == "2" ]
then

# Pre-processing:
#----------------
# BP:
sac << END
r *HZ*SAC 
rmean
rtrend
bp co $bp1 $bp2 p 2
w append bp
quit
END

# running absolute mean normalization + whitening:
# (only for the classical cross-correlation)
#-------------------------------------------------
for f in *HZ*SACbp
do
$norm $f N=0 white
done

# Bandpass whitened data:
#------------------------
# BP:
sac << END
r *HZ*SACbpn
rmean
rtrend
bp co $bp1 $bp2 p 2
w over
quit
END

# Correlation loops:
#-------------------
n1=`ls -1 *HZ*bpn | wc -l | awk '{ print $1}'`
# start n1 while-loop 
while [ "$n1" -ge "1" ]
do

f1=` ls -1 *HZ*bpn | tail -$n1 | head -1 ` 
f3=` echo $f1 | sed 's/bpn/bp/' `
n2=` echo $n1 | awk '{ print $1-1}' ` 

#start n2 while loop:
while [ "$n2" -ge "1" ]
do

f2=` ls -1 *HZ*bpn | tail -$n2 | head -1 ` 
f4=` echo $f2 | sed 's/bpn/bp/' `

sac <<- %
rh $f1 $f2
setbb a &1,b
setbb b &2,b
setbb c &1,e
setbb d &2,e
cut o ( (max %a %b%) + 1 ) ( (min %c %d%) - 1 )
r $f1 $f2
setbb e &1,npts
setbb f &2,npts
ch npts (min %e %f%)
write d1.sac d2.sac
cut off
rh $f3 $f4
setbb a &1,b
setbb b &2,b
setbb c &1,e
setbb d &2,e
cut o ( (max %a %b%) + 1 ) ( (min %c %d%) - 1 )
r $f3 $f4
write d3.sac d4.sac
quit
%

# cross-correlate time freq - normalized data (CCGN):
# (this can also be computed with sac!)
#========================================================
echo $pcc CCGN
$pcc d1.sac d2.sac isac osac ccgn nl1=-500 nl2=500 nn nsmin=4000
\rm -f d1.sac d2.sac t.t

sta1=`echo $f1 | awk '{ print substr($1,16,4)}' `
sta2=`echo $f2 | awk '{ print substr($1,16,4)}' `
dat=`echo $f1 | awk '{ print substr($1,1,15)}' `
paar="$sta1"Z_"$sta2"Z
paar0="$dir0"/"$paar"

if test -d $paar0
then
mv ccgn.sac $paar0"/"$dat"ccgn_"$paar".sacn"
else
mkdir $paar0
mv ccgn.sac $paar0"/"$dat"ccgn_"$paar".sacn"
fi

# cross-correlate data without time freq - normalization (PCC):
#===============================================================
echo $pcc PCC
$pcc d3.sac d4.sac isac osac pcc nl1=-500 nl2=500 nn pov=1 nsmin=4000
mv pcc.sac $paar0"/"$dat"pcc1_"$paar".sac"

n2=`echo $n2 | awk '{ print $1-1}' ` 
#end n2 while-loop
done

n1=`echo $n1 | awk '{ print $1-1}' `
#end n1 while-loop
done 

# clean up 
\rm -f *SACbp*
\rm -f d?.sac
fi
# end data for-loop:
done

exit
